package com.bpcl.lpg.core.cronjob;

import org.slf4j.LoggerFactory;
import de.hybris.platform.cronjob.model.CronJobModel;
import java.nio.file.Path;
import java.nio.file.Files;
import java.nio.file.StandardOpenOption;
import java.nio.file.OpenOption;
import java.nio.file.Paths;
import java.io.Closeable;
import org.apache.zookeeper.common.IOUtils;
import org.apache.commons.lang.StringUtils;
import com.jcraft.jsch.SftpException;
import com.jcraft.jsch.JSchException;
import java.io.IOException;
import de.hybris.platform.util.Config;
import java.util.Objects;
import java.io.Writer;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.File;
import java.util.TimeZone;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;
import de.hybris.platform.core.servicelayer.data.SearchPageData;
import java.util.Date;
import de.hybris.platform.cronjob.enums.CronJobStatus;
import de.hybris.platform.cronjob.enums.CronJobResult;
import java.text.MessageFormat;
import com.bpcl.core.util.BPCLDateUtils;
import de.hybris.platform.servicelayer.exceptions.ModelSavingException;
import com.bpcl.lpg.core.model.CashMemoModel;
import de.hybris.platform.tx.Transaction;
import java.util.Collection;
import org.apache.commons.collections.CollectionUtils;
import de.hybris.platform.servicelayer.cronjob.PerformResult;
import com.bpcl.core.ym.notication.service.LpgV2NotificationEngineService;
import com.bpcl.lpg.core.service.LpgDeliveryConfirmationService;
import com.bpcl.lpg.core.encryption.pgp.DefaultLpgPgpKeys;
import javax.annotation.Resource;
import de.hybris.platform.servicelayer.model.ModelService;
import com.bpcl.core.ym.notication.service.NotificationEngineService;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import com.bpcl.lpg.core.dao.LpgDeliveryConfirmationDao;
import org.slf4j.Logger;
import com.bpcl.core.model.BpclCronjobModel;
import de.hybris.platform.servicelayer.cronjob.AbstractJobPerformable;

public class LpgCashMemoExportJob extends AbstractJobPerformable<BpclCronjobModel>
{
    private static final Logger LOG;
    private static final String namePrefix = "BPCL_CashMemo_";
    private static final String IST = "IST";
    private static final String COD = "COD";
    private static final String ONLINE = "Online Payment";
    private static final String COD_DIGITAL = "COD Digital";
    private static final String SEPARETOR = "~";
    public static final String TEMP_DIR;
    private static final String FILE_TYPE = ".csv";
    private static final String TEMP = "temp_";
    private LpgDeliveryConfirmationDao lpgDeliveryConfirmationDao;
    private ConfigurationService configurationService;
    private NotificationEngineService notificationEngineService;
    @Resource(name = "modelService")
    private ModelService modelService;
    @Resource(name = "defaultLpgPgpKeys")
    private DefaultLpgPgpKeys defaultLpgPgpKeys;
    private LpgDeliveryConfirmationService lpgDeliveryConfirmationService;
    private LpgV2NotificationEngineService lpgV2NotificationEngineService;
    
    public PerformResult perform(final BpclCronjobModel cronjob) {
        final Date startDate = this.populateStartDate(cronjob);
        LpgCashMemoExportJob.LOG.info("Date ranges starts from : {}", startDate);
        final Date endDate = this.populateEndDate(cronjob);
        LpgCashMemoExportJob.LOG.info("Filter End Date : {}", endDate);
        int currentPage = 0;
        try {
            final int pageSize = this.configurationService.getConfiguration().getInt("bpcl.lpg.delivery.confirmation.outbound.page.size", 25000);
            SearchPageData<CashMemoModel> searchPageData = (SearchPageData<CashMemoModel>)this.lpgDeliveryConfirmationDao.getCashMemoDetailsForExport(startDate, endDate, pageSize, currentPage);
            final int totalPages = searchPageData.getPagination().getNumberOfPages();
            LpgCashMemoExportJob.LOG.info("Page size: {}, total pages: {}, total records: {}", pageSize, totalPages, searchPageData.getPagination().getTotalNumberOfResults());
            while (currentPage < totalPages) {
                searchPageData = (SearchPageData<CashMemoModel>)this.lpgDeliveryConfirmationDao.getCashMemoDetailsForExport(startDate, endDate, pageSize, 0);
                final List<CashMemoModel> cashMemoList = searchPageData.getResults();
                LpgCashMemoExportJob.LOG.info("cashMemoList {} ", (Object)(CollectionUtils.isNotEmpty((Collection)cashMemoList) ? cashMemoList.size() : 0));
                if (CollectionUtils.isNotEmpty((Collection)cashMemoList)) {
                    this.generateReportCSV(cashMemoList);
                    final Transaction txn = Transaction.current();
                    txn.begin();
                    boolean success = false;
                    try {
                        for (final CashMemoModel cashMemo : cashMemoList) {
                            LpgCashMemoExportJob.LOG.info("cashMemoPK {} ", cashMemo);
                            LpgCashMemoExportJob.LOG.info("Cash Memo {} Sent to LPGNEXT ", cashMemo.getCashMemoSerialNumber());
                            cashMemo.setSentToLPGNxt(Boolean.valueOf(true));
                        }
                        this.modelService.saveAll((Collection)cashMemoList);
                        success = true;
                    }
                    catch (final ModelSavingException e) {
                        LpgCashMemoExportJob.LOG.error("Exception Occured While updating CashMemo sentToLpgNxt flag :" + e.getMessage());
                        throw new ModelSavingException("Exception Occured While updating CashMemo");
                    }
                    finally {
                        if (success) {
                            txn.commit();
                        }
                        else {
                            txn.rollback();
                        }
                    }
                }
                ++currentPage;
            }
        }
        catch (final Exception ex) {
            LpgCashMemoExportJob.LOG.error("CSV file Processing Exception : ", ex);
            final String body = MessageFormat.format("Cash Memo Export cronjob is failed due to exception at {0}", BPCLDateUtils.getCurrentDateAsString("dd-MMM-yy h:mm a"));
            this.lpgV2NotificationEngineService.sendEmail("FAILED Cash Memo Export Job Alert", body, this.configurationService.getConfiguration().getString("lpg.cash.memo.alert.email.receiver.id"));
            return new PerformResult(CronJobResult.FAILURE, CronJobStatus.ABORTED);
        }
        return new PerformResult(CronJobResult.SUCCESS, CronJobStatus.FINISHED);
    }
    
    private Date populateStartDate(final BpclCronjobModel cronjob) {
        Date startDate = cronjob.getFilterStartDate();
        if (startDate == null) {
            final Calendar cal = Calendar.getInstance();
            cal.add(10, -this.configurationService.getConfiguration().getInt("lpg.cash.memo.export.data.start.period"));
            startDate = cal.getTime();
        }
        return startDate;
    }
    
    private Date populateEndDate(final BpclCronjobModel cronjob) {
        Date endDate = cronjob.getFilterEndDate();
        if (endDate == null) {
            final Calendar cal = Calendar.getInstance();
            cal.add(12, -10);
            endDate = cal.getTime();
        }
        return endDate;
    }
    
    private void generateReportCSV(final List<CashMemoModel> cashMemoList) {
        final SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMMdd_HHmmss");
        simpleDateFormat.setTimeZone(TimeZone.getTimeZone("IST"));
        final String date = simpleDateFormat.format(new Date());
        final String fileExtension = ".csv";
        final String fileName = "BPCL_CashMemo_" + date + fileExtension;
        final String tempFileName = LpgCashMemoExportJob.TEMP_DIR + "temp_" + fileName;
        final String path = this.configurationService.getConfiguration().getString("bpcl.cashmemo.hybris.input.filepath") + fileName;
        File cashMemoFile = new File(path);
        final StringBuilder fileContant = new StringBuilder();
        try {
            final BufferedWriter writer = new BufferedWriter(new FileWriter(path));
            try {
                this.writeCsvHeader(fileContant);
                fileContant.append(System.getProperty("line.separator"));
                for (final CashMemoModel cashMemo : cashMemoList) {
                    this.writeCsvLine(fileContant, cashMemo);
                }
                LpgCashMemoExportJob.LOG.info("CSV File created Successfully and file name is {}", fileName);
                if (Objects.nonNull(fileContant)) {
                    createFile(tempFileName, fileContant.toString());
                }
                final File tempOrigFile = new File(tempFileName);
                LpgCashMemoExportJob.LOG.info("Inside generateReportCSV tempOrigFile : {} ", Objects.nonNull(tempOrigFile) ? tempOrigFile.getAbsolutePath() : "");
                final String csvPath = this.configurationService.getConfiguration().getString("bpcl.cashmemo.hybris.input.filepath") + "BPCL_CashMemo_" + simpleDateFormat.format(new Date()) + fileExtension;
                LpgCashMemoExportJob.LOG.info("Encrypted CSV File path is {}", csvPath);
                final File encryptedFile = new File(csvPath);
                if (Config.getBoolean("lpg.encryption.enable", false)) {
                    LpgCashMemoExportJob.LOG.info("Encrypting the file");
                    if (Objects.nonNull(tempOrigFile)) {
                        LpgCashMemoExportJob.LOG.info("tempOrigFile not null");
                        this.defaultLpgPgpKeys.fileEncryption(encryptedFile, tempOrigFile.getAbsolutePath(), this.configurationService.getConfiguration().getString("lpg.encryption.publickey"));
                    }
                    cashMemoFile = encryptedFile;
                }
                final String targetPath = this.configurationService.getConfiguration().getString("lpg.sftp.delivery.confirmation.outbound.path");
                this.lpgDeliveryConfirmationService.uploadFileOnSFTP(String.valueOf(encryptedFile), targetPath);
                LpgCashMemoExportJob.LOG.info("CSV File sent to SFTP Successfully and file name is {}", fileName);
                deleteFile(String.valueOf(tempOrigFile));
                writer.close();
            }
            catch (final Throwable t) {
                try {
                    writer.close();
                }
                catch (final Throwable exception) {
                    t.addSuppressed(exception);
                }
                throw t;
            }
        }
        catch (final IOException | JSchException | SftpException e) {
            throw new RuntimeException(e);
        }
    }
    
    private void writeCsvHeader(final StringBuilder writer) throws IOException {
        writer.append("DistCode~RefillId~ConsumerNumber~LPGId~ConsumerId~CashMemoNo~DeliveryDate~PaymentMode~DeliveryStatus~UndeliveryReason~ReferenceId~AmountReceived~ScheduleDate~CashMemoSerialNo~Latitude~Longitude~DeliveryPersonName~DeliveryPersonMobile~OTPCode");
    }
    
    private void writeCsvLineA(final BufferedWriter writer, final CashMemoModel cashMemo) throws IOException {
        final String separator = "~";
        LpgCashMemoExportJob.LOG.info("Inside writeCsvLine ");
        writer.write(String.valueOf(cashMemo.getDistCode()));
        LpgCashMemoExportJob.LOG.info("Inside writeCsvLine getDistCode {} ", cashMemo.getDistCode());
        writer.write(separator);
        writer.write(String.valueOf(cashMemo.getRefillId()));
        writer.write(separator);
        writer.write(String.valueOf(cashMemo.getConsumerNumber()));
        writer.write(separator);
        LpgCashMemoExportJob.LOG.info("Inside writeCsvLine RefillId : {} ConsumerNumber : {} ", cashMemo.getRefillId(), cashMemo.getConsumerNumber());
        writer.write(String.valueOf(cashMemo.getLpgId()));
        writer.write(separator);
        writer.write(String.valueOf(cashMemo.getConsumerId()));
        writer.write(separator);
        writer.write(cashMemo.getCashMemoNo());
        writer.write(separator);
        LpgCashMemoExportJob.LOG.info("Inside writeCsvLine LpgId : {} ConsumerId : {} CashMemoNo : {} ", cashMemo.getLpgId(), cashMemo.getConsumerId(), cashMemo.getCashMemoNo());
        final String deliveryDate = BPCLDateUtils.dateToString(cashMemo.getDeliveryDate(), "yyyy-MM-dd HH:mm:ss");
        writer.write(deliveryDate);
        writer.write(separator);
        if ("COD".equalsIgnoreCase(cashMemo.getPaymentMode())) {
            writer.write("1");
        }
        else if ("Online Payment".equalsIgnoreCase(cashMemo.getPaymentMode())) {
            writer.write("2");
        }
        else if ("COD Digital".equalsIgnoreCase(cashMemo.getPaymentMode())) {
            writer.write("20");
        }
        else {
            writer.write("0");
        }
        writer.write(separator);
        writer.write(String.valueOf(cashMemo.getDeliveryStatus()));
        writer.write(separator);
        if (cashMemo.getUnDeliveredReason() == null) {
            writer.write("0");
        }
        else {
            writer.write(String.valueOf(cashMemo.getUnDeliveredReason()));
        }
        writer.write(separator);
        writer.write(cashMemo.getReferenceId());
        writer.write(separator);
        writer.write(String.valueOf(cashMemo.getCashMemoAmount()));
        writer.write(separator);
        LpgCashMemoExportJob.LOG.info("Inside writeCsvLine DeliveryStatus : {} UnDeliveredReason : {} CashMemoAmount : {} ", cashMemo.getDeliveryStatus(), cashMemo.getUnDeliveredReason(), cashMemo.getCashMemoAmount());
        final String scheduleDate = BPCLDateUtils.dateToString(cashMemo.getScheduleDate(), "yyyy-MM-dd");
        writer.write(scheduleDate);
        writer.write(separator);
        writer.write(String.valueOf(cashMemo.getCashMemoSerialNumber()));
        writer.write(separator);
        if (StringUtils.isNotBlank(cashMemo.getLatitude())) {
            writer.write(cashMemo.getLatitude());
        }
        writer.write(separator);
        if (StringUtils.isNotBlank(cashMemo.getLongitude())) {
            writer.write(cashMemo.getLongitude());
        }
        writer.write(separator);
        writer.write(cashMemo.getOperatorName());
        writer.write(separator);
        writer.write(cashMemo.getOperatorMobileNumber());
        writer.write(separator);
        writer.write(String.valueOf(cashMemo.getDeliveryOTPCode()));
        writer.write("\n");
        LpgCashMemoExportJob.LOG.info("Inside writeCsvLine OperatorName : {} OperatorMobileNumber : {} DeliveryOTPCode : {} ", cashMemo.getOperatorName(), cashMemo.getOperatorMobileNumber(), cashMemo.getDeliveryOTPCode());
    }
    
    private void writeCsvLine(final StringBuilder stringBuilder, final CashMemoModel cashMemo) throws IOException {
        LpgCashMemoExportJob.LOG.info("Inside writeCsvLine ");
        stringBuilder.append(String.valueOf(cashMemo.getDistCode()));
        stringBuilder.append("~");
        stringBuilder.append(String.valueOf(cashMemo.getRefillId()));
        stringBuilder.append("~");
        stringBuilder.append(String.valueOf(cashMemo.getConsumerNumber()));
        stringBuilder.append("~");
        stringBuilder.append(String.valueOf(cashMemo.getLpgId()));
        stringBuilder.append("~");
        stringBuilder.append(String.valueOf(cashMemo.getConsumerId()));
        stringBuilder.append("~");
        stringBuilder.append(cashMemo.getCashMemoNo());
        stringBuilder.append("~");
        final String deliveryDate = BPCLDateUtils.dateToString(cashMemo.getDeliveryDate(), "yyyy-MM-dd HH:mm:ss");
        stringBuilder.append(deliveryDate);
        stringBuilder.append("~");
        LpgCashMemoExportJob.LOG.info("Inside writeCsvLine LpgId : {} ConsumerId : {} CashMemoNo : {}  PaymentMode : {}", cashMemo.getLpgId(), cashMemo.getConsumerId(), cashMemo.getCashMemoNo(), cashMemo.getPaymentMode());
        if ("COD".equalsIgnoreCase(cashMemo.getPaymentMode())) {
            stringBuilder.append("1");
        }
        else if ("Online Payment".equalsIgnoreCase(cashMemo.getPaymentMode())) {
            stringBuilder.append("2");
        }
        else if ("COD Digital".equalsIgnoreCase(cashMemo.getPaymentMode())) {
            stringBuilder.append("20");
        }
        else {
            stringBuilder.append("0");
        }
        stringBuilder.append("~");
        stringBuilder.append(String.valueOf(cashMemo.getDeliveryStatus()));
        stringBuilder.append("~");
        if (cashMemo.getUnDeliveredReason() == null) {
            stringBuilder.append("0");
        }
        else {
            stringBuilder.append(String.valueOf(cashMemo.getUnDeliveredReason()));
        }
        stringBuilder.append("~");
        stringBuilder.append(cashMemo.getReferenceId());
        stringBuilder.append("~");
        stringBuilder.append(String.valueOf(cashMemo.getCashMemoAmount()));
        stringBuilder.append("~");
        final String scheduleDate = BPCLDateUtils.dateToString(cashMemo.getScheduleDate(), "yyyy-MM-dd");
        stringBuilder.append(scheduleDate);
        stringBuilder.append("~");
        stringBuilder.append(String.valueOf(cashMemo.getCashMemoSerialNumber()));
        stringBuilder.append("~");
        if (StringUtils.isNotBlank(cashMemo.getLatitude())) {
            stringBuilder.append(cashMemo.getLatitude());
        }
        stringBuilder.append("~");
        if (StringUtils.isNotBlank(cashMemo.getLongitude())) {
            stringBuilder.append(cashMemo.getLongitude());
        }
        stringBuilder.append("~");
        stringBuilder.append(cashMemo.getOperatorName());
        stringBuilder.append("~");
        stringBuilder.append(cashMemo.getOperatorMobileNumber());
        stringBuilder.append("~");
        stringBuilder.append(String.valueOf(cashMemo.getDeliveryOTPCode()));
        stringBuilder.append(System.getProperty("line.separator"));
    }
    
    public static void createFile(final String fileName, final String fileContent) {
        FileWriter writer = null;
        try {
            writer = new FileWriter(fileName);
            writer.write(fileContent);
        }
        catch (final IOException e) {
            LpgCashMemoExportJob.LOG.error("Exception occured while creating file : " + e);
        }
        finally {
            IOUtils.closeStream((Closeable)writer);
        }
    }
    
    public static void deleteFile(final String fileDir) throws IOException {
        final Path path = Paths.get(fileDir, new String[0]);
        Files.newOutputStream(path, StandardOpenOption.DELETE_ON_CLOSE);
    }
    
    public boolean isAbortable() {
        return true;
    }
    
    public void setConfigurationService(final ConfigurationService configurationService) {
        this.configurationService = configurationService;
    }
    
    public void setNotificationEngineService(final NotificationEngineService notificationEngineService) {
        this.notificationEngineService = notificationEngineService;
    }
    
    public void setLpgDeliveryConfirmationDao(final LpgDeliveryConfirmationDao lpgDeliveryConfirmationDao) {
        this.lpgDeliveryConfirmationDao = lpgDeliveryConfirmationDao;
    }
    
    public void setLpgDeliveryConfirmationService(final LpgDeliveryConfirmationService lpgDeliveryConfirmationService) {
        this.lpgDeliveryConfirmationService = lpgDeliveryConfirmationService;
    }
    
    public LpgV2NotificationEngineService getLpgV2NotificationEngineService() {
        return this.lpgV2NotificationEngineService;
    }
    
    public void setLpgV2NotificationEngineService(final LpgV2NotificationEngineService lpgV2NotificationEngineService) {
        this.lpgV2NotificationEngineService = lpgV2NotificationEngineService;
    }
    
    static {
        LOG = LoggerFactory.getLogger(LpgCashMemoExportJob.class);
        TEMP_DIR = Config.getParameter("bpcl.cashmemo.hybris.input.filepath");
    }
}
